/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2002 The Ant-Contrib project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Ant", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package net.sf.antcontrib.cpptasks;

import java.io.File;

import net.sf.antcontrib.cpptasks.compiler.ProcessorConfiguration;

/**
 * A description of a file built or to be built
 */
public final class TargetInfo
{
     private /* final */ ProcessorConfiguration config;
     private /* final */ File[] sources;
     private File[] sysSources;
     private /* final */ File output;
     private boolean rebuild;
     private static final File[] emptyFileArray = new File[0];

     public TargetInfo(ProcessorConfiguration config,
        File[] sources,
        File[] sysSources,
        File output,
        boolean rebuild) {
        if(config == null) {
            throw new NullPointerException("config");
        }
        if(sources == null) {
            throw new NullPointerException("sources");
        }
        if(output == null) {
            throw new NullPointerException("output");
        }
        this.config = config;
        this.sources = (File[]) sources.clone();
        if(sysSources == null) {
          this.sysSources = emptyFileArray;
        } else {
          this.sysSources = (File[]) sysSources.clone();
        }
        this.output = output;
        this.rebuild = rebuild;
        //
        //   if the output doesn't exist, must rebuild it
        //
        if(!output.exists()) {
          rebuild = true;
        }
     }

    public File[] getSources() {
       File[] clone = (File[]) sources.clone();
	   return clone;
    }

    public String[] getSourcePaths() {
      String[] paths = new String[sources.length];
      for(int i = 0; i < sources.length; i++) {
        paths[i] = sources[i].toString();
      }
      return paths;
    }

    public File[] getSysSources() {
       File[] clone = (File[]) sysSources.clone();
	   return clone;
    }

    public String[] getSysSourcePaths() {
      String[] paths = new String[sysSources.length];
      for(int i = 0; i < sysSources.length; i++) {
        paths[i] = sysSources[i].toString();
      }
      return paths;
    }

    public File[] getAllSources() {
      File[] allSources = new File[sources.length + sysSources.length];
      for (int i = 0; i < sysSources.length; i++) {
        allSources[i] = sysSources[i];
      }
      int offset = sysSources.length;
      for (int i = 0; i < sources.length; i++) {
        allSources[i + offset] = sources[i];
      }
      return allSources;
    }

    public String[] getAllSourcePaths() {
      String[] paths = new String[sysSources.length + sources.length];
      for(int i = 0; i < sysSources.length; i++) {
        paths[i] = sysSources[i].toString();
      }
      int offset = sysSources.length;
      for (int i = 0; i < sources.length; i++) {
        paths[offset + i] = sources[i].toString();
      }
      return paths;
    }


    public File getOutput() {
      return output;
    }

    public void mustRebuild() {
      this.rebuild = true;
    }

    public boolean getRebuild() {
      return rebuild;
    }

    public ProcessorConfiguration getConfiguration() {
      return config;
    }

    /**
     * Returns an array of SourceHistory objects (contains relative path
     * and last modified time) for the source[s] of this target
     */
    public SourceHistory[] getSourceHistories(String basePath) {
       SourceHistory[] histories = new SourceHistory[sources.length];
       for(int i = 0; i < sources.length; i++) {
          String relativeName = CUtil.getRelativePath(basePath, sources[i]);
          long lastModified = sources[i].lastModified();
          histories[i] = new SourceHistory(relativeName,lastModified);
        }
        return histories;
    }
}
