/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2002 The Ant-Contrib project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Ant", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package net.sf.antcontrib.cpptasks.borland;

import java.io.File;
import java.util.Vector;

import net.sf.antcontrib.cpptasks.compiler.CommandLineCompilerConfiguration;
import net.sf.antcontrib.cpptasks.compiler.CompilerConfiguration;
import net.sf.antcontrib.cpptasks.compiler.LinkType;
import net.sf.antcontrib.cpptasks.compiler.Linker;
import net.sf.antcontrib.cpptasks.compiler.PrecompilingCommandLineCCompiler;

/**
 * Adapter for the Borland(r) C/C++ compiler.
 *
 * @author Curt Arnold
 */
public class BorlandCCompiler extends PrecompilingCommandLineCCompiler
{
    private static final BorlandCCompiler instance = new BorlandCCompiler();

    private BorlandCCompiler() {
      super("bcc32", "-?",  new String[] { ".c", ".cc", ".cpp", ".cxx", ".c++" },
            new String[] { ".h", ".hpp", ".inl" }, ".obj", false, null );
    }


    public static BorlandCCompiler getInstance() {
      return instance;
    }

    protected File[] getEnvironmentIncludePath() {
        return BorlandProcessor.getEnvironmentPath("bcc32",'I',
          new String[] { "..\\include" });
    }

    protected String getIncludeDirSwitch(String includeDir) {
      return BorlandProcessor.getIncludeDirSwitch("-I",includeDir);
    }
    protected void addWarningSwitch(Vector args, int level) {
      BorlandProcessor.addWarningSwitch(args,level);
    }


    protected void getDefineSwitch(StringBuffer buffer,String define, String value) {
      BorlandProcessor.getDefineSwitch(buffer,define,value);
    }

    protected void getUndefineSwitch(StringBuffer buffer, String define) {
      BorlandProcessor.getUndefineSwitch(buffer,define);
    }

    protected void addImpliedArgs(Vector args,
      boolean debug,
      boolean multithreaded,
      boolean exceptions,
      LinkType linkType) {
      args.addElement("-c");
      //
      //  turn off compiler autodependency since
      //     we do it ourselves
      args.addElement("-X");
      if(exceptions) {
        args.addElement("-x");
      }
      else {
        args.addElement("-x-");
      }
      if(multithreaded) {
        args.addElement("-tWM");
      }
      if(debug) {
        args.addElement("-Od");
        args.addElement("-v");
      }
    }


    public Linker getLinker(LinkType type) {
      return BorlandLinker.getInstance().getLinker(type);
    }


   protected CompilerConfiguration createPrecompileGeneratingConfig(
      CommandLineCompilerConfiguration baseConfig, File prototype,
      String lastInclude) {
      String[] additionalArgs = new String[] {
          "-H=" + lastInclude,
          "-Hc" };
      return new CommandLineCompilerConfiguration(baseConfig,
          additionalArgs, null, true);
   }

   protected CompilerConfiguration createPrecompileUsingConfig(
      CommandLineCompilerConfiguration baseConfig, File prototype,
      String lastInclude, String[] exceptFiles) {

      String[] additionalArgs = new String[] {
          "-Hu" };
      return new CommandLineCompilerConfiguration(baseConfig,
          additionalArgs, exceptFiles, false);
   }

}
