/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Ant-Contrib project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Ant", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package net.sf.antcontrib.cpptasks.compiler;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.util.Vector;

import org.apache.tools.ant.taskdefs.Execute;
import org.apache.tools.ant.taskdefs.ExecuteStreamHandler;

/**
 * Implements ExecuteStreamHandler to capture the
 * output of a Execute to an array of strings
 * @author Curt Arnold
 */
public class CaptureStreamHandler implements ExecuteStreamHandler
{
  private InputStream fromProcess;
  private InputStream errorStream;

  /**
   * Runs an executable and captures the output in a String array
   *
   * @param cmdline command line arguments
   * @return output of process
   */
  public static String[] run(String[] cmdline) {
    CaptureStreamHandler handler = new CaptureStreamHandler();
    Execute exec = new Execute(handler);
    exec.setCommandline(cmdline);
    try {
      int status = exec.execute();
    } catch(IOException ex) {
    }
    return handler.getOutput();
  }

  public CaptureStreamHandler() {
  }

    /**
     * Install a handler for the input stream of the subprocess.
     *
     * @param os output stream to write to the standard input stream of the
     *           subprocess
     */
    public void setProcessInputStream(OutputStream os) throws IOException {
      os.close();
    }

    /**
     * Install a handler for the error stream of the subprocess.
     *
     * @param is input stream to read from the error stream from the subprocess
     */
    public void setProcessErrorStream(InputStream is) throws IOException {
      errorStream = is;
    }


    /**
     * Install a handler for the output stream of the subprocess.
     *
     * @param is input stream to read from the error stream from the subprocess
     */
    public void setProcessOutputStream(InputStream is) throws IOException {
      fromProcess = is;
    }

    /**
     * Start handling of the streams.
     */
    public void start() throws IOException {
    }

    /**
     * Stop handling of the streams - will not be restarted.
     */
    public void stop() {
    }

    public String[] getOutput() {
      String[] output;
      if (fromProcess != null) {
        Vector lines = new Vector(10);
        try {
          BufferedReader reader = new BufferedReader(new InputStreamReader(errorStream));
          for (int i = 0; i < 2; i++) {
            for (int j = 0; j < 100; j++) {
              String line = reader.readLine();
              if (line == null) {
                reader = new BufferedReader(new InputStreamReader(fromProcess));
                break;
              }
              lines.addElement(line);
            }
          }
        } catch(IOException ex) {
        }
        output = new String[lines.size()];
        lines.copyInto(output);
        return output;
      }
      output = new String[0];
      return output;
    }


}
