/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Ant-Contrib project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Ant", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package net.sf.antcontrib.cpptasks.devstudio;

import java.io.File;
import java.util.Vector;

import net.sf.antcontrib.cpptasks.CUtil;
import net.sf.antcontrib.cpptasks.compiler.CommandLineLinker;
import net.sf.antcontrib.cpptasks.compiler.LinkType;



/**
 * Abstract base class for linkers that try to mimic the
 * command line arguments for the Microsoft (r) Incremental Linker
 *
 * @author Curt Arnold
 */
public abstract class DevStudioCompatibleLinker extends CommandLineLinker
{
    public DevStudioCompatibleLinker(String command,
        String identifierArg, String outputSuffix)
    {
        super(command, identifierArg,
            new String[] { ".obj", ".lib",".res" },
          new String[] { ".map", ".pdb", ".lnk", ".dll" },
          outputSuffix, false, null);
    }

    public File[] getLibraryPath() {
        return CUtil.getPathFromEnvironment("LIB",";");
    }

    protected void addFixed(Boolean fixed, Vector args) {
      if(fixed != null) {
        if(fixed.booleanValue()) {
          args.addElement("/FIXED");
        }
        else {
          args.addElement("/FIXED:NO");
        }
      }
    }

    protected void addIncremental(boolean incremental, Vector args) {
      if(incremental) {
          args.addElement("/INCREMENTAL:YES");
        }
        else {
          args.addElement("/INCREMENTAL:NO");
        }
    }

    protected void addMap(boolean map, Vector args) {
      if(map) {
          args.addElement("/MAP");
      }
    }

    protected void addBase(long base, Vector args) {
      if(base >= 0) {
        String baseAddr = Long.toHexString(base);
        args.addElement("/BASE:0x" + baseAddr);
      }
    }

    protected void addStack(int stack, Vector args) {
      if(stack >= 0) {
        String stackStr = Integer.toHexString(stack);
        args.addElement("/STACK:0x" + stackStr);
      }
    }


    protected void addImpliedArgs(boolean debug,
      LinkType linkType, Vector args) {
      args.addElement("/NOLOGO");
      if (debug) {
       args.addElement("/DEBUG");
      }
      if(linkType.isSharedLibrary()) {
        args.addElement("/DLL");
      }
/*
      if(linkType.isSubsystemGUI()) {
        args.addElement("/SUBSYSTEM:WINDOWS");
      } else {
        if(linkType.isSubsystemConsole()) {
          args.addElement("/SUBSYSTEM:CONSOLE");
        }
      }
*/
    }

    public String getCommandFileSwitch(String commandFile) {
      return "@" + commandFile;
    }

    public String[] getOutputFileSwitch(String outputFile) {
      return new String[] { "/OUT:" + outputFile };
    }

    public int getMaximumCommandLength() {
      return 1024;
    }

    public String[] getLibraryPatterns(String[] libnames) {
      StringBuffer buf = new StringBuffer();
      String[] patterns = new String[libnames.length];
      for (int i = 0; i < libnames.length; i++) {
        buf.setLength(0);
        buf.append(libnames[i]);
        buf.append(".lib");
        patterns[i] = buf.toString();
      }
      return patterns;
    }

    public boolean isCaseSensitive() {
      return false;
    }

}
