/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2002 The Ant-Contrib project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Ant", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package net.sf.antcontrib.cpptasks.devstudio;

import java.io.File;
import java.util.Vector;

import net.sf.antcontrib.cpptasks.CUtil;
import net.sf.antcontrib.cpptasks.compiler.CommandLineCompiler;
import net.sf.antcontrib.cpptasks.compiler.LinkType;
import net.sf.antcontrib.cpptasks.compiler.Linker;
import net.sf.antcontrib.cpptasks.parser.CParser;
import net.sf.antcontrib.cpptasks.parser.Parser;

/**
 * Adapter for the Microsoft (r) Windows 32 Resource Compiler
 *
 * @author Curt Arnold
 */
public final class DevStudioResourceCompiler extends CommandLineCompiler
{
    private static final DevStudioResourceCompiler instance =
      new DevStudioResourceCompiler();

    private String identifier;

    private DevStudioResourceCompiler() {
      super("rc", null,  new String[] { ".rc" },
            new String[] { ".h", ".hpp", ".inl" }, ".res",
            false, null );
    }


    public static DevStudioResourceCompiler getInstance() {
      return instance;
    }

    /**
     * The include parser for C will work just fine,
     * but we didn't want to inherit from CommandLineCCompiler
     */
    protected Parser createParser() {
      return new CParser();
    }

	protected int getMaximumInputFilesPerCommand() {
		return 1;
	}
	protected int getArgumentCountPerInputFile() {
		return 2;
	}
	protected int getTotalArgumentLengthForInputFile(File outputDir,String inputFile) {
		String arg1 = getInputFileArgument(outputDir,inputFile,0);
		String arg2 = getInputFileArgument(outputDir,inputFile,1);
		return arg1.length() + arg2.length() + 2;
	}
	protected String getInputFileArgument(File outputDir,String filename, int index) {
		if(index == 0) {
			String outputFileName = getOutputFileName(filename);
			String fullOutputName = new File(outputDir,outputFileName).toString();
			return "/fo" + fullOutputName;
		}
		return filename;
	}


    protected File[] getEnvironmentIncludePath() {
        return CUtil.getPathFromEnvironment("INCLUDE",";");
    }

    protected String getIncludeDirSwitch(String includeDir) {
	  return DevStudioProcessor.getIncludeDirSwitch(includeDir);
    }

    protected void getDefineSwitch(StringBuffer buffer,String define, String value) {
	   DevStudioProcessor.getDefineSwitch(buffer,define,value);
    }

    protected void getUndefineSwitch(StringBuffer buffer, String define) {
      DevStudioProcessor.getUndefineSwitch(buffer,define);
    }
    protected void addWarningSwitch(Vector args, int level) {
    }


    protected void addImpliedArgs(Vector args,
      boolean debug,
      boolean multithreaded,
      boolean exceptions,
      LinkType linkType) {
      if(debug) {
        args.addElement("/D_DEBUG");
      }
      else {
        args.addElement("/DNDEBUG");
      }
    }



    public Linker getLinker(LinkType type) {
      return DevStudioLinker.getInstance().getLinker(type);
    }

}
