/*
* The Apache Software License, Version 1.1
*
* Copyright (c) 2002 The Ant-Contrib project.  All rights
* reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*
* 3. The end-user documentation included with the redistribution, if
*    any, must include the following acknowlegement:
*       "This product includes software developed by the
*        Apache Software Foundation (http://www.apache.org/)."
*    Alternately, this acknowlegement may appear in the software itself,
*    if and wherever such third-party acknowlegements normally appear.
*
* 4. The names "The Jakarta Project", "Ant", and "Apache Software
*    Foundation" must not be used to endorse or promote products derived
*    from this software without prior written permission. For written
*    permission, please contact apache@apache.org.
*
* 5. Products derived from this software may not be called "Apache"
*    nor may "Apache" appear in their names without prior written
*    permission of the Apache Group.
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
* ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
* OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGE.
* ====================================================================
*
* This software consists of voluntary contributions made by many
* individuals on behalf of the Apache Software Foundation.  For more
* information on the Apache Software Foundation, please see
* <http://www.apache.org/>.
*/
package net.sf.antcontrib.cpptasks.os400;

import net.sf.antcontrib.cpptasks.compiler.CommandLineCCompiler;
import net.sf.antcontrib.cpptasks.compiler.LinkType;
import net.sf.antcontrib.cpptasks.CUtil;
import net.sf.antcontrib.cpptasks.compiler.CaptureStreamHandler;
import org.apache.tools.ant.BuildException;
import net.sf.antcontrib.cpptasks.CompilerDef;
import net.sf.antcontrib.cpptasks.compiler.CompilerConfiguration;
import net.sf.antcontrib.cpptasks.compiler.CommandLineCompilerConfiguration;
import net.sf.antcontrib.cpptasks.types.CommandLineArgument;
import net.sf.antcontrib.cpptasks.compiler.Linker;
import net.sf.antcontrib.cpptasks.compiler.AbstractCompiler;
import java.util.Hashtable;

import java.io.File;
import java.util.*;

/**
* Adapter for the IBM (R) OS/390 (tm) C++ Compiler
*
* @author Hiram Chirino (cojonudo14@hotmail.com)
*/
public class IccCompiler extends CommandLineCCompiler {
	private static final AbstractCompiler instance = new IccCompiler();

	private IccCompiler() {
		super("icc", null,
			new String[] { ".c", ".cc", ".cpp", ".cxx", ".c++", ".s" },
			new String[] { ".h", ".hpp"}, ".o", false, null);
	}

	public static AbstractCompiler getInstance() {
		return instance;
	}

	protected File[] getEnvironmentIncludePath() {
		return CUtil.getPathFromEnvironment("INCLUDE", ":");
	}

	protected String getIncludeDirSwitch(String includeDir) {
		return IccProcessor.getIncludeDirSwitch(includeDir);
	}
	protected void addWarningSwitch(Vector args, int level) {
		IccProcessor.addWarningSwitch(args, level);
	}

	protected void addImpliedArgs(
		Vector args,
		boolean debug,
		boolean multithreaded,
		boolean exceptions,
                LinkType linkType) {

		// Specifies that only compilations and assemblies be done.
                //  Link-edit is not done
		args.addElement("-c");

		/*
		if (exceptions) {
			args.addElement("/GX");
		}
		*/

		if (debug) {
			args.addElement("-g");
			/*
			args.addElement("-D");
			args.addElement("_DEBUG");
			if (multithreaded) {
				args.addElement("/D_MT");
				if (staticLink) {
					args.addElement("/MTd");
				} else {
					args.addElement("/MDd");
					args.addElement("/D_DLL");
				}
			} else {
				args.addElement("/MLd");
			}
			*/
		} else {
			/*
			args.addElement("-D");
			args.addElement("NEBUG");
			if (multithreaded) {
				args.addElement("/D_MT");
				if (staticLink) {
					args.addElement("/MT");
				} else {
					args.addElement("/MD");
					args.addElement("/D_DLL");
				}
			} else {
				args.addElement("/ML");
			}
			*/
		}

	}

	public Linker getLinker(LinkType type) {
          return IccLinker.getInstance().getLinker(type);
	}

	/* Only compile one file at time for now */
    protected int getMaximumInputFilesPerCommand() {
    	return 1;
      //return Integer.MAX_VALUE;
    }

	/*
	 * @see CommandLineCompiler#getDefineSwitch(StringBuffer, String, String)
	 */
	protected void getDefineSwitch(StringBuffer buffer, String define, String value) {
	   buffer.append("-q");
       buffer.append(define);
	   if(value != null && value.length() > 0) {
	      buffer.append('=');
	      buffer.append(value);
	   }
	}

	/*
	 * @see CommandLineCompiler#getUndefineSwitch(StringBuffer, String)
	 */
	protected void getUndefineSwitch(StringBuffer buffer, String define) {
	   /*
	   buffer.addElement("-q");
       buf.append(define);
       */
	}

}
