/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2002 The Ant-Contrib project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Ant", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package net.sf.antcontrib.cpptasks.parser;

import java.io.IOException;
import java.io.Reader;
import java.util.Vector;

/**
 * A parser that extracts #include statements from a Reader.
 *
 * @author Adam Murdoch
 * @author Curt Arnold
 */
public final class CParser extends AbstractParser implements Parser
{
    private AbstractParserState newLineState;
    private final Vector includes = new Vector();


    /**
     *
     *
     */
    public CParser() {
        AbstractParserState quote = new FilenameState(this,new char[] { '"' });
        AbstractParserState bracket = new FilenameState(this,new char[] { '>' });
        AbstractParserState postE = new PostE(this,bracket,quote);
        //
        //    nclude
        //
        AbstractParserState e = new LetterState(this,'e',postE,null);
        AbstractParserState d = new LetterState(this,'d',e,null);
        AbstractParserState u = new LetterState(this,'u',d,null);
        AbstractParserState l = new LetterState(this,'l',u,null);
        AbstractParserState c = new LetterState(this,'c',l,null);
        AbstractParserState n = new LetterState(this,'n',c,null);

        //
        //   mport is equivalent to nclude
        //
        AbstractParserState t = new LetterState(this, 't', postE, null);
        AbstractParserState r = new LetterState(this, 'r', t, null);
        AbstractParserState o = new LetterState(this, 'o', r, null);
        AbstractParserState p = new LetterState(this, 'p', o, null);
        AbstractParserState m = new LetterState(this, 'm', p, null);

        //
        //   switch between
        //
        AbstractParserState n_m = new BranchState(this,
          new char[] { 'n', 'm' },
          new AbstractParserState[] { n, m },
          null);

        AbstractParserState i = new WhitespaceOrLetterState(this,'i',n_m);
        newLineState = new LetterState(this,'#',i,null);
    }

    public void parse(Reader reader) throws IOException {
        includes.setSize(0);
        super.parse(reader);
    }

    public String[] getIncludes() {
        String[] retval = new String[includes.size()];
        includes.copyInto(retval);
        return retval;
    }


    public void addFilename(String include) {
      includes.addElement(include);
    }

    public AbstractParserState getNewLineState() {
      return newLineState;
    }


}

