/*
* The Apache Software License, Version 1.1
*
* Copyright (c) 2002 The Ant-Contrib project.  All rights
* reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*
* 3. The end-user documentation included with the redistribution, if
*    any, must include the following acknowlegement:
*       "This product includes software developed by the
*        Apache Software Foundation (http://www.apache.org/)."
*    Alternately, this acknowlegement may appear in the software itself,
*    if and wherever such third-party acknowlegements normally appear.
*
* 4. The names "The Jakarta Project", "Ant", and "Apache Software
*    Foundation" must not be used to endorse or promote products derived
*    from this software without prior written permission. For written
*    permission, please contact apache@apache.org.
*
* 5. Products derived from this software may not be called "Apache"
*    nor may "Apache" appear in their names without prior written
*    permission of the Apache Group.
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
* ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
* OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGE.
* ====================================================================
*
* This software consists of voluntary contributions made by many
* individuals on behalf of the Apache Software Foundation.  For more
* information on the Apache Software Foundation, please see
* <http://www.apache.org/>.
*/
package net.sf.antcontrib.cpptasks.sun;

import java.io.File;
import java.util.Vector;

import net.sf.antcontrib.cpptasks.CCTask;
import net.sf.antcontrib.cpptasks.CUtil;
import net.sf.antcontrib.cpptasks.compiler.CommandLineLinker;
import net.sf.antcontrib.cpptasks.compiler.LinkType;
import net.sf.antcontrib.cpptasks.compiler.Linker;
import net.sf.antcontrib.cpptasks.types.LibrarySet;



/**
* Adapter for the IBM (R) OS/390 (tm) Linker
*
* @author Hiram Chirino (cojonudo14@hotmail.com)
*/
public final class C89Linker extends CommandLineLinker
{
    private static final C89Linker instance =
        new C89Linker("", "");
    private static final C89Linker dllLinker =
        new C89Linker("lib", ".so");

    private String outputPrefix;

    private C89Linker(String outputPrefix, String outputSuffix)
    {
        super("ld", "/bogus", new String[] { ".o", ".a", ".lib", ".x" },
          new String[] { }, outputSuffix, false, null);
        this.outputPrefix = outputPrefix;
    }

    public static C89Linker getInstance() {
      return instance;
    }

    public File[] getLibraryPath() {
        return CUtil.getPathFromEnvironment("LIB",";");
    }

    public String getOutputFileName(String baseName) {
        return outputPrefix + super.getOutputFileName(baseName);
    }

    protected void addFixed(Boolean fixed, Vector args) {
    }


    protected void addMap(boolean map, Vector args) {
    }

    protected void addBase(long base, Vector args) {
    }

    protected void addIncremental(boolean incremental, Vector args) {
    }


    protected void addStack(int stack, Vector args) {
    }


    public Linker getLinker(LinkType linkType) {
      if(linkType.isSharedLibrary()) {
        return dllLinker;
      }
      /*
      if(linkType.isStaticLibrary()) {
        return OS390Librarian.getInstance();
      }
      */
      return instance;
    }

    protected void addImpliedArgs(boolean debug, LinkType linkType, Vector args) {
        if(linkType.isSharedLibrary()) {
          args.addElement("-G");
        }
    }

    public String getCommandFileSwitch(String commandFile) {
      return "@" + commandFile;
    }

    public String[] getOutputFileSwitch(String outputFile) {
      return new String[] { "-o", outputFile };
    }

    public int getMaximumCommandLength() {
      return Integer.MAX_VALUE;
    }

    public String[] getLibraryPatterns(String[] libnames) {
      return C89Processor.getLibraryPatterns(libnames);
    }

    public boolean isCaseSensitive() {
      return C89Processor.isCaseSensitive();
    }
    
    public String[] addLibrarySets (CCTask task, LibrarySet[] libsets, Vector preargs,
        Vector midargs, Vector endargs) {
      super.addLibrarySets(task, libsets,preargs, midargs, endargs);
      StringBuffer buf = new StringBuffer("-l");
      for (int i = 0; i < libsets.length; i++) {
        LibrarySet set = libsets[i];
        File libdir = set.getDir(null);
        String[] libs = set.getLibs();
        if(libdir != null) {
          endargs.addElement("-L");
          endargs.addElement(libdir.getAbsolutePath());
        }
        for (int j = 0; j < libs.length; j++) {
          //
          //  reset the buffer to just "-l"
          //
          buf.setLength(2);
          //
          //  add the library name
          buf.append(libs[j]);
          //
          //  add the argument to the list
          endargs.addElement(buf.toString());
        }
      }
      return null;
    }    
}

